@@ -7,17 +7,19 @@
  *	eventually be moved into other files.
  *
  * Copyright (c) 1995-1997 Sun Microsystems, Inc.
- * Copyright 2001, Apple Computer, Inc.
- * Copyright (c) 2005-2007 Daniel A. Steffen <das@users.sourceforge.net>
+ * Copyright 2001-2009, Apple Inc.
+ * Copyright (c) 2005-2009 Daniel A. Steffen <das@users.sourceforge.net>
+ * Copyright 2014 Marc Culler.
  *
- * See the file "license.terms" for information on usage and redistribution of
- * this file, and for a DISCLAIMER OF ALL WARRANTIES.
+ * See the file "license.terms" for information on usage and redistribution
+ * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  */
 
 #include "tkMacOSXPrivate.h"
 #include "tkMacOSXEvent.h"
 
 #include <IOKit/IOKitLib.h>
+#include <IOKit/hidsystem/IOHIDShared.h>
 
 /*
  * Because this file is still under major development Debugger statements are
@@ -31,6 +33,9 @@
 
 #define ROOT_ID 10
 
+CGFloat tkMacOSXZeroScreenHeight = 0;
+CGFloat tkMacOSXZeroScreenTop = 0;
+
 /*
  * Declarations of static variables used in this file.
  */
@@ -54,13 +59,8 @@ static int		DefaultErrorHandler(Display *display,
 
 static int		DestroyImage(XImage *image);
 static unsigned long	ImageGetPixel(XImage *image, int x, int y);
-static int		PutPixel(XImage *image, int x, int y,
+static int		ImagePutPixel(XImage *image, int x, int y,
 			    unsigned long pixel);
-#if 0
-static XImage *		SubImage(XImage *image, int x, int y,
-			    unsigned int width, unsigned int height);
-static int		AddPixel(XImage *image, long value);
-#endif
 
 /*
  *----------------------------------------------------------------------
@@ -83,38 +83,35 @@ void
 TkMacOSXDisplayChanged(
     Display *display)
 {
-    GDHandle graphicsDevice;
     Screen *screen;
-    Rect bounds = {0, 0, 0, 0}, *maxBounds;
+    NSArray *nsScreens;
+
 
     if (display == NULL || display->screens == NULL) {
 	return;
     }
     screen = display->screens;
 
-    graphicsDevice = GetMainDevice();
-    screen->root_depth = (*(*graphicsDevice)->gdPMap)->cmpSize *
-	    (*(*graphicsDevice)->gdPMap)->cmpCount;
-    screen->height = (*graphicsDevice)->gdRect.bottom -
-	    (*graphicsDevice)->gdRect.top;
-    screen->width = (*graphicsDevice)->gdRect.right -
-	    (*graphicsDevice)->gdRect.left;
+    nsScreens = [NSScreen screens];
+    if (nsScreens && [nsScreens count]) {
+	NSScreen *s = [nsScreens objectAtIndex:0];
+	NSRect bounds = [s frame], visible = [s visibleFrame];
+	NSRect maxBounds = NSZeroRect;
 
-    screen->mwidth = (screen->width * 254 + 360) / 720;
-    screen->mheight = (screen->height * 254 + 360) / 720;
+	tkMacOSXZeroScreenHeight = bounds.size.height;
+	tkMacOSXZeroScreenTop = tkMacOSXZeroScreenHeight -
+		(visible.origin.y + visible.size.height);
 
-    maxBounds = (Rect *) screen->ext_data;
-    *maxBounds = bounds;
-    graphicsDevice = GetDeviceList();
-    while (graphicsDevice) {
-	OSStatus err;
+	screen->root_depth = NSBitsPerPixelFromDepth([s depth]);
+	screen->width = bounds.size.width;
+	screen->height = bounds.size.height;
+	screen->mwidth = (bounds.size.width * 254 + 360) / 720;
+	screen->mheight = (bounds.size.height * 254 + 360) / 720;
 
-	err = ChkErr(GetAvailableWindowPositioningBounds, graphicsDevice,
-		&bounds);
-	if (err == noErr) {
-	    UnionRect(&bounds, maxBounds, maxBounds);
+	for (s in nsScreens) {
+	    maxBounds = NSUnionRect(maxBounds, [s visibleFrame]);
 	}
-	graphicsDevice = GetNextDevice(graphicsDevice);
+	*((NSRect *)screen->ext_data) = maxBounds;
     }
 }
 
@@ -137,12 +134,15 @@ TkMacOSXDisplayChanged(
 
 TkDisplay *
 TkpOpenDisplay(
-    CONST char *display_name)
+    const char *display_name)
 {
     Display *display;
     Screen *screen;
     int fd = 0;
-    static Rect maxBounds = {0, 0, 0, 0};
+    static NSRect maxBounds = {{0, 0}, {0, 0}};
+    static char vendor[25] = "";
+    NSArray *cgVers;
+    NSAutoreleasePool *pool = [NSAutoreleasePool new];
 
     if (gMacDisplay != NULL) {
 	if (strcmp(gMacDisplay->display->display_name, display_name) == 0) {
@@ -152,8 +152,8 @@ TkpOpenDisplay(
 	}
     }
 
-    display = (Display *) ckalloc(sizeof(Display));
-    screen  = (Screen *) ckalloc(sizeof(Screen));
+    display = ckalloc(sizeof(Display));
+    screen  = ckalloc(sizeof(Screen));
     bzero(display, sizeof(Display));
     bzero(screen, sizeof(Screen));
 
@@ -166,11 +166,35 @@ TkpOpenDisplay(
     display->default_screen = 0;
     display->display_name   = (char *) macScreenName;
 
-    Gestalt(gestaltQuickdrawVersion, (long *) &display->proto_minor_version);
-    display->proto_major_version = 10;
-    display->proto_minor_version -= gestaltMacOSXQD;
-    display->vendor = "Apple";
-    Gestalt(gestaltSystemVersion, (long *) &display->release);
+    cgVers = [[[NSBundle bundleWithIdentifier:@"com.apple.CoreGraphics"]
+	    objectForInfoDictionaryKey:@"CFBundleShortVersionString"]
+	    componentsSeparatedByString:@"."];
+    if ([cgVers count] >= 2) {
+	display->proto_major_version = [[cgVers objectAtIndex:1] integerValue];
+    }
+    if ([cgVers count] >= 3) {
+	display->proto_minor_version = [[cgVers objectAtIndex:2] integerValue];
+    }
+    if (!vendor[0]) {
+	snprintf(vendor, sizeof(vendor), "Apple AppKit %g",
+		NSAppKitVersionNumber);
+    }
+    display->vendor = vendor;
+    {
+	int major, minor, patch;
+
+#if MAC_OS_X_VERSION_MIN_REQUIRED < 10100
+	Gestalt(gestaltSystemVersionMajor, (SInt32*)&major);
+	Gestalt(gestaltSystemVersionMinor, (SInt32*)&minor);
+	Gestalt(gestaltSystemVersionBugFix, (SInt32*)&patch);
+#else
+	NSOperatingSystemVersion systemVersion = [[NSProcessInfo processInfo] operatingSystemVersion];
+	major = systemVersion.majorVersion;
+	minor = systemVersion.minorVersion;
+	patch = systemVersion.patchVersion;
+#endif
+	display->release = major << 16 | minor << 8 | patch;
+    }
 
     /*
      * These screen bits never change
@@ -181,7 +205,7 @@ TkpOpenDisplay(
     screen->white_pixel = 0x00FFFFFF | PIXEL_MAGIC << 24;
     screen->ext_data	= (XExtData *) &maxBounds;
 
-    screen->root_visual = (Visual *) ckalloc(sizeof(Visual));
+    screen->root_visual = ckalloc(sizeof(Visual));
     screen->root_visual->visualid     = 0;
     screen->root_visual->class	      = TrueColor;
     screen->root_visual->red_mask     = 0x00FF0000;
@@ -196,7 +220,7 @@ TkpOpenDisplay(
 
     TkMacOSXDisplayChanged(display);
 
-    gMacDisplay = (TkDisplay *) ckalloc(sizeof(TkDisplay));
+    gMacDisplay = ckalloc(sizeof(TkDisplay));
 
     /*
      * This is the quickest way to make sure that all the *Init flags get
@@ -205,6 +229,7 @@ TkpOpenDisplay(
 
     bzero(gMacDisplay, sizeof(TkDisplay));
     gMacDisplay->display = display;
+    [pool drain];
     return gMacDisplay;
 }
 
@@ -237,11 +262,11 @@ TkpCloseDisplay(
     gMacDisplay = NULL;
     if (display->screens != NULL) {
 	if (display->screens->root_visual != NULL) {
-	    ckfree((char *) display->screens->root_visual);
+	    ckfree(display->screens->root_visual);
 	}
-	ckfree((char *) display->screens);
+	ckfree(display->screens);
     }
-    ckfree((char *) display);
+    ckfree(display);
 }
 
 /*
@@ -272,7 +297,7 @@ TkClipCleanup(
      * needed.
      */
 
-    TkSuspendClipboard();
+    [NSApp tkProvidePasteboard:dispPtr];
 
     if (dispPtr->clipWindow != NULL) {
 	Tk_DeleteSelHandler(dispPtr->clipWindow, dispPtr->clipboardAtom,
@@ -318,33 +343,6 @@ MacXIdAlloc(
     return ++cur_id;
 }
 
-/*
- *----------------------------------------------------------------------
- *
- * TkpWindowWasRecentlyDeleted --
- *
- *	Tries to determine whether the given window was recently deleted.
- *	Called from the generic code error handler to attempt to deal with
- *	async BadWindow errors under some circumstances.
- *
- * Results:
- *	Always 0, we do not keep this information on the Mac, so we do not
- *	know whether the window was destroyed.
- *
- * Side effects:
- *	None.
- *
- *----------------------------------------------------------------------
- */
-
-int
-TkpWindowWasRecentlyDeleted(
-    Window win,
-    TkDisplay *dispPtr)
-{
-    return 0;
-}
-
 /*
  *----------------------------------------------------------------------
  *
@@ -432,14 +430,11 @@ XGetGeometry(
 	*border_width_return = winPtr->changes.border_width;
 	*depth_return = Tk_Depth(winPtr);
     } else {
-	Rect boundsRect;
-	CGrafPtr destPort = TkMacOSXGetDrawablePort(d);
-
-	GetPortBounds(destPort, &boundsRect);
-	*x_return = boundsRect.left;
-	*y_return =  boundsRect.top;
-	*width_return = boundsRect.right - boundsRect.left;
-	*height_return = boundsRect.bottom - boundsRect.top;
+	CGSize size = ((MacDrawable *) d)->size;
+	*x_return = 0;
+	*y_return =  0;
+	*width_return = size.width;
+	*height_return = size.height;
 	*border_width_return = 0;
 	*depth_return = 32;
     }
@@ -474,7 +469,7 @@ XBell(
     Display* display,
     int percent)
 {
-    SysBeep(percent);
+    NSBeep();
     return Success;
 }
 
@@ -653,7 +648,8 @@ XGetWindowProperty(
 }
 
 void
-XRefreshKeyboardMapping( XMappingEvent* x)
+XRefreshKeyboardMapping(
+    XMappingEvent *x)
 {
     /* used by tkXEvent.c */
     Debugger();
@@ -718,7 +714,7 @@ XSetClipRectangles(
 
     while (n--) {
 	XRectangle rect = *rectangles;
-	
+
 	rect.x += clip_x_origin;
 	rect.y += clip_y_origin;
 	TkUnionRectWithRegion(&rect, clipRgn, clipRgn);
@@ -755,17 +751,20 @@ TkGetServerInfo(
     Tk_Window tkwin)		/* Token for window; this selects a particular
 				 * display and server. */
 {
-    char buffer[8 + TCL_INTEGER_SPACE * 2];
-    char buffer2[TCL_INTEGER_SPACE];
+    char buffer[5 + TCL_INTEGER_SPACE * 2];
+    char buffer2[11 + TCL_INTEGER_SPACE];
 
-    sprintf(buffer, "QD%dR%x ", ProtocolVersion(Tk_Display(tkwin)),
+    snprintf(buffer, sizeof(buffer), "CG%d.%d ",
+	    ProtocolVersion(Tk_Display(tkwin)),
 	    ProtocolRevision(Tk_Display(tkwin)));
-    sprintf(buffer2, " %x", VendorRelease(Tk_Display(tkwin)));
+    snprintf(buffer2, sizeof(buffer2), " Mac OS X %x",
+	    VendorRelease(Tk_Display(tkwin)));
     Tcl_AppendResult(interp, buffer, ServerVendor(Tk_Display(tkwin)),
 	    buffer2, NULL);
 }
 
 #pragma mark XImage handling
+
 /*
  *----------------------------------------------------------------------
  *
@@ -796,9 +795,8 @@ XCreateImage(
     int bytes_per_line)
 {
     XImage *ximage;
-
     display->request++;
-    ximage = (XImage *) ckalloc(sizeof(XImage));
+    ximage = ckalloc(sizeof(XImage));
 
     ximage->height = height;
     ximage->width = width;
@@ -806,6 +804,11 @@ XCreateImage(
     ximage->xoffset = offset;
     ximage->format = format;
     ximage->data = data;
+    ximage->obdata = NULL;
+    /* The default pixelpower is 0.  This must be explicitly set to 1 in the
+     * case of an XImage extracted from a Retina display.
+     */
+    ximage->pixelpower = 0;
 
     if (format == ZPixmap) {
 	ximage->bits_per_pixel = 32;
@@ -837,11 +840,10 @@ XCreateImage(
     ximage->red_mask = 0x00FF0000;
     ximage->green_mask = 0x0000FF00;
     ximage->blue_mask = 0x000000FF;
-    ximage->obdata = NULL;
     ximage->f.create_image = NULL;
     ximage->f.destroy_image = DestroyImage;
     ximage->f.get_pixel = ImageGetPixel;
-    ximage->f.put_pixel = PutPixel;
+    ximage->f.put_pixel = ImagePutPixel;
     ximage->f.sub_image = NULL;
     ximage->f.add_pixel = NULL;
 
@@ -856,8 +858,10 @@ XCreateImage(
  *	This function copies data from a pixmap or window into an XImage.
  *
  * Results:
- *	Returns a newly allocated image containing the data from the given
- *	rectangle of the given drawable.
+ *	Returns a newly allocated XImage containing the data from the given
+ *	rectangle of the given drawable, or NULL if the XImage could not be
+ *	constructed.  NOTE: If we are copying from a window on a Retina
+ *	display, the dimensions of the XImage will be 2*width x 2*height.
  *
  * Side effects:
  *	None.
@@ -876,52 +880,102 @@ XGetImage(
     unsigned long plane_mask,
     int format)
 {
-    XImage *   imagePtr = NULL;
-    Pixmap     pixmap = (Pixmap) NULL;
-    Tk_Window  win = (Tk_Window) ((MacDrawable *) d)->winPtr;
-    GC	       gc;
-    int	       depth = 32;
-    int	       offset = 0;
-    int	       bitmap_pad = 32;
-    int	       bytes_per_line = 0;
+    NSBitmapImageRep *bitmap_rep;
+    NSUInteger         bitmap_fmt;
+    XImage *       imagePtr = NULL;
+    char *           bitmap = NULL;
+    char *           image_data=NULL;
+    int	        depth = 32;
+    int	        offset = 0;
+    int	        bitmap_pad = 0;
+    int	        bytes_per_row = 4*width;
+    int                size;
+    MacDrawable *macDraw = (MacDrawable *) d;
+    NSWindow *win = TkMacOSXDrawableWindow(d);
+    /* This code assumes that backing scale factors are integers.  Currently
+     * Retina displays use a scale factor of 2.0 and normal displays use 1.0.
+     * We do not support any other values here.
+     */
+    int scalefactor = 1;
+    if (win && [win respondsToSelector:@selector(backingScaleFactor)]) { 
+	scalefactor = ([win backingScaleFactor] == 2.0) ? 2 : 1;
+    }
+    int scaled_height = height * scalefactor;
+    int scaled_width = width * scalefactor;
 
-    if (TkMacOSXGetDrawablePort(d)) {
-	if (format == ZPixmap) {
-	    if (width > 0 && height > 0) {
-		/*
-		 * Tk_GetPixmap fails for zero width or height.
-		 */
-
-		pixmap = Tk_GetPixmap(display, d, width, height, depth);
-	    }
-	    if (win) {
-		XGCValues values;
-
-		gc = Tk_GetGC(win, 0, &values);
-	    } else {
-		gc = XCreateGC(display, pixmap, 0, NULL);
-	    }
-	    if (pixmap) {
-		XCopyArea(display, d, pixmap, gc, x, y, width, height, 0, 0);
-	    }
-	    imagePtr = XCreateImage(display, NULL, depth, format, offset,
-		    (char *) TkMacOSXGetDrawablePort(pixmap),
-		    width, height, bitmap_pad, bytes_per_line);
-
-	    /*
-	     * Track Pixmap underlying the XImage in the unused obdata field
-	     * so that we can treat XImages coming from XGetImage specially.
-	     */
-
-	    imagePtr->obdata = (XPointer) pixmap;
-	    if (!win) {
-		XFreeGC(display, gc);
-	    }
-	} else {
-	    TkpDisplayWarning(
-		    "XGetImage: only ZPixmap types are implemented",
-		    "XGetImage Failure");
+    if (format == ZPixmap) {
+	if (width == 0 || height == 0) {
+	    /* This happens all the time.
+	    TkMacOSXDbgMsg("XGetImage: empty image requested");
+	    */
+	    return NULL;
 	}
+
+	bitmap_rep =  BitmapRepFromDrawableRect(d, x, y, width, height);
+	bitmap_fmt = [bitmap_rep bitmapFormat];
+
+	if ( bitmap_rep == Nil                        ||
+	     (bitmap_fmt != 0 && bitmap_fmt != 1)     ||
+	     [bitmap_rep samplesPerPixel] != 4 ||
+	     [bitmap_rep isPlanar] != 0               ) {
+	    TkMacOSXDbgMsg("XGetImage: Failed to construct NSBitmapRep");
+	    return NULL;
+	}
+
+	NSSize image_size = NSMakeSize(width, height);
+	NSImage* ns_image = [[NSImage alloc]initWithSize:image_size];
+	[ns_image addRepresentation:bitmap_rep];
+
+	/* Assume premultiplied nonplanar data with 4 bytes per pixel.*/
+	if ( [bitmap_rep isPlanar ] == 0 &&
+	     [bitmap_rep samplesPerPixel] == 4 ) {
+	    bytes_per_row = [bitmap_rep bytesPerRow];
+	    assert(bytes_per_row == 4 * scaled_width);
+	    assert([bitmap_rep bytesPerPlane] == bytes_per_row * scaled_height);
+	    size = bytes_per_row*scaled_height;
+	    image_data = (char*)[bitmap_rep bitmapData];
+	    if ( image_data ) {
+		int row, n, m;
+		bitmap = ckalloc(size);
+		/*
+		  Oddly enough, the bitmap has the top row at the beginning,
+		  and the pixels are in BGRA or ABGR format.
+		*/
+		if (bitmap_fmt == 0) {
+		    /* BGRA */
+		    for (row=0, n=0; row<scaled_height; row++, n+=bytes_per_row) {
+			for (m=n; m<n+bytes_per_row; m+=4) {
+			    *(bitmap+m)   = *(image_data+m+2);
+			    *(bitmap+m+1) = *(image_data+m+1);
+			    *(bitmap+m+2) = *(image_data+m);
+			    *(bitmap+m+3) = *(image_data+m+3);
+			}
+		    }
+		} else {
+		    /* ABGR */
+		    for (row=0, n=0; row<scaled_height; row++, n+=bytes_per_row) {
+			for (m=n; m<n+bytes_per_row; m+=4) {
+			    *(bitmap+m)   = *(image_data+m+3);
+			    *(bitmap+m+1) = *(image_data+m+2);
+			    *(bitmap+m+2) = *(image_data+m+1);
+			    *(bitmap+m+3) = *(image_data+m);
+			}
+		    }
+		}
+	    }
+	}
+	if (bitmap) {
+	    imagePtr = XCreateImage(display, NULL, depth, format, offset,
+				    (char*)bitmap, scaled_width, scaled_height,
+				    bitmap_pad, bytes_per_row);
+	    if (scalefactor == 2) {
+	    	imagePtr->pixelpower = 1;
+	     }
+	    [ns_image removeRepresentation:bitmap_rep]; /*releases the rep*/
+	    [ns_image release];
+	}
+    } else {
+	TkMacOSXDbgMsg("Could not extract image from drawable.");
     }
     return imagePtr;
 }
@@ -947,12 +1001,10 @@ DestroyImage(
     XImage *image)
 {
     if (image) {
-	if (image->obdata) {
-	    Tk_FreePixmap((Display*) gMacDisplay, (Pixmap) image->obdata);
-	} else if (image->data) {
+	if (image->data) {
 	    ckfree(image->data);
 	}
-	ckfree((char*) image);
+	ckfree(image);
     }
     return 0;
 }
@@ -979,38 +1031,57 @@ ImageGetPixel(
     int x,
     int y)
 {
-    CGrafPtr destPort, savePort;
-    Boolean portChanged;
-    RGBColor cPix;
-    unsigned long r, g, b, c;
+    unsigned char r = 0, g = 0, b = 0;
 
-    destPort = (CGrafPtr) image->data;
-    portChanged = QDSwapPort(destPort, &savePort);
-    GetCPixel(x, y, &cPix);
-    if (image->obdata) {
-	/*
-	 * Image from XGetImage, 16 bit color values.
-	 */
+    if (image && image->data) {
+	unsigned char *srcPtr = ((unsigned char*) image->data)
+		+ (y * image->bytes_per_line)
+		+ (((image->xoffset + x) * image->bits_per_pixel) / NBBY);
 
-	r = (cPix . red) >> 8;
-	g = (cPix . green) >> 8;
-	b = (cPix . blue) >> 8;
-    } else {
-	r = cPix . red;
-	g = cPix . green;
-	b = cPix . blue;
+	switch (image->bits_per_pixel) {
+	    case 32: {
+		r = (*((unsigned int*) srcPtr) >> 16) & 0xff;
+		g = (*((unsigned int*) srcPtr) >>  8) & 0xff;
+		b = (*((unsigned int*) srcPtr)      ) & 0xff;
+		/*if (image->byte_order == LSBFirst) {
+		    r = srcPtr[2]; g = srcPtr[1]; b = srcPtr[0];
+		} else {
+		    r = srcPtr[1]; g = srcPtr[2]; b = srcPtr[3];
+		}*/
+		break;
+	    }
+	    case 16:
+		r = (*((unsigned short*) srcPtr) >> 7) & 0xf8;
+		g = (*((unsigned short*) srcPtr) >> 2) & 0xf8;
+		b = (*((unsigned short*) srcPtr) << 3) & 0xf8;
+		break;
+	    case 8:
+		r = (*srcPtr << 2) & 0xc0;
+		g = (*srcPtr << 4) & 0xc0;
+		b = (*srcPtr << 6) & 0xc0;
+		r |= r >> 2 | r >> 4 | r >> 6;
+		g |= g >> 2 | g >> 4 | g >> 6;
+		b |= b >> 2 | b >> 4 | b >> 6;
+		break;
+	    case 4: {
+		unsigned char c = (x % 2) ? *srcPtr : (*srcPtr >> 4);
+		r = (c & 0x04) ? 0xff : 0;
+		g = (c & 0x02) ? 0xff : 0;
+		b = (c & 0x01) ? 0xff : 0;
+		break;
+		}
+	    case 1:
+		r = g = b = ((*srcPtr) & (0x80 >> (x % 8))) ? 0xff : 0;
+		break;
+	}
     }
-    c = (r<<16)|(g<<8)|(b);
-    if (portChanged) {
-	QDSwapPort(savePort, NULL);
-    }
-    return c;
+    return (PIXEL_MAGIC << 24) | (r << 16) | (g << 8) | b;
 }
 
 /*
  *----------------------------------------------------------------------
  *
- * PutPixel --
+ * ImagePutPixel --
  *
  *	Set a single pixel in an image.
  *
@@ -1024,64 +1095,53 @@ ImageGetPixel(
  */
 
 static int
-PutPixel(
+ImagePutPixel(
     XImage *image,
     int x,
     int y,
     unsigned long pixel)
 {
-    CGrafPtr destPort, savePort;
-    Boolean portChanged;
-    RGBColor cPix;
-    unsigned long r, g, b;
+    if (image && image->data) {
+	unsigned char r = ((pixel & image->red_mask)   >> 16) & 0xff;
+	unsigned char g = ((pixel & image->green_mask) >>  8) & 0xff;
+	unsigned char b = ((pixel & image->blue_mask)       ) & 0xff;
+	unsigned char *dstPtr = ((unsigned char*) image->data)
+		+ (y * image->bytes_per_line)
+		+ (((image->xoffset + x) * image->bits_per_pixel) / NBBY);
 
-    destPort = (CGrafPtr)image->data;
-    portChanged = QDSwapPort(destPort, &savePort);
-    r = (pixel & image->red_mask)>>16;
-    g = (pixel & image->green_mask)>>8;
-    b = (pixel & image->blue_mask);
-    if (image->obdata) {
-	/*
-	 * Image from XGetImage, 16 bit color values.
-	 */
-
-	cPix.red = r << 8;
-	cPix.green = g << 8;
-	cPix.blue = b << 8;
-    } else {
-	cPix.red = r;
-	cPix.green = g;
-	cPix.blue = b;
-    }
-    SetCPixel(x, y, &cPix);
-    if (portChanged) {
-	QDSwapPort(savePort, NULL);
+	switch (image->bits_per_pixel) {
+	    case 32:
+		*((unsigned int*) dstPtr) = (0xff << 24) | (r << 16) |
+			(g << 8) | b;
+		/*if (image->byte_order == LSBFirst) {
+		    dstPtr[3] = 0xff; dstPtr[2] = r; dstPtr[1] = g; dstPtr[0] = b;
+		} else {
+		    dstPtr[0] = 0xff; dstPtr[1] = r; dstPtr[2] = g; dstPtr[3] = b;
+		}*/
+		break;
+	    case 16:
+		*((unsigned short*) dstPtr) = ((r & 0xf8) << 7) |
+			((g & 0xf8) << 2) | ((b & 0xf8) >> 3);
+		break;
+	    case 8:
+		*dstPtr = ((r & 0xc0) >> 2) | ((g & 0xc0) >> 4) |
+			((b & 0xc0) >> 6);
+		break;
+	    case 4: {
+		unsigned char c = ((r & 0x80) >> 5) | ((g & 0x80) >> 6) |
+			((b & 0x80) >> 7);
+		*dstPtr = (x % 2) ? ((*dstPtr & 0xf0) | (c & 0x0f)) :
+			((*dstPtr & 0x0f) | ((c << 4) & 0xf0));
+		break;
+		}
+	    case 1:
+		*dstPtr = ((r|g|b) & 0x80) ? (*dstPtr | (0x80 >> (x % 8))) :
+			(*dstPtr & ~(0x80 >> (x % 8)));
+		break;
+	}
     }
     return 0;
 }
-
-#if 0
-static XImage *
-SubImage(
-    XImage *image,
-    int x,
-    int y,
-    unsigned int width,
-    unsigned int height)
-{
-    Debugger();
-    return NULL;
-}
-
-static int
-AddPixel(
-    XImage *image,
-    long value)
-{
-    Debugger();
-    return 0;
-}
-#endif
 
 /*
  *----------------------------------------------------------------------
@@ -1104,24 +1164,24 @@ AddPixel(
 
 void
 XChangeWindowAttributes(
-    Display* display,
+    Display *display,
     Window w,
     unsigned long value_mask,
-    XSetWindowAttributes* attributes)
+    XSetWindowAttributes *attributes)
 {
 }
 
 void
 XSetWindowBackground(
-	Display *display,
-	Window window,
-	unsigned long value)
+    Display *display,
+    Window window,
+    unsigned long value)
 {
 }
 
 void
 XSetWindowBackgroundPixmap(
-    Display* display,
+    Display *display,
     Window w,
     Pixmap background_pixmap)
 {
@@ -1129,7 +1189,7 @@ XSetWindowBackgroundPixmap(
 
 void
 XSetWindowBorder(
-    Display* display,
+    Display *display,
     Window w,
     unsigned long border_pixel)
 {
@@ -1137,7 +1197,7 @@ XSetWindowBorder(
 
 void
 XSetWindowBorderPixmap(
-    Display* display,
+    Display *display,
     Window w,
     Pixmap border_pixmap)
 {
@@ -1145,7 +1205,7 @@ XSetWindowBorderPixmap(
 
 void
 XSetWindowBorderWidth(
-    Display* display,
+    Display *display,
     Window w,
     unsigned int width)
 {
@@ -1153,7 +1213,7 @@ XSetWindowBorderWidth(
 
 void
 XSetWindowColormap(
-    Display* display,
+    Display *display,
     Window w,
     Colormap colormap)
 {
@@ -1162,24 +1222,25 @@ XSetWindowColormap(
 
 Status
 XStringListToTextProperty(
-    char** list,
+    char **list,
     int count,
-    XTextProperty* text_prop_return)
+    XTextProperty *text_prop_return)
 {
     Debugger();
     return (Status) 0;
 }
+
 void
 XSetWMClientMachine(
-    Display* display,
+    Display *display,
     Window w,
-    XTextProperty* text_prop)
+    XTextProperty *text_prop)
 {
     Debugger();
 }
+
 XIC
-XCreateIC(
-    void)
+XCreateIC(void)
 {
     Debugger();
     return (XIC) 0;
@@ -1202,10 +1263,10 @@ XCreateIC(
  *----------------------------------------------------------------------
  */
 
-CONST char *
+const char *
 TkGetDefaultScreenName(
     Tcl_Interp *interp,		/* Not used. */
-    CONST char *screenName)		/* If NULL, use default string. */
+    const char *screenName)		/* If NULL, use default string. */
 {
 #if 0
     if ((screenName == NULL) || (screenName[0] == '\0')) {
@@ -1306,5 +1367,35 @@ void
 Tk_ResetUserInactiveTime(
     Display *dpy)
 {
-    UpdateSystemActivity(UsrActivity);
+    IOGPoint loc;
+    kern_return_t kr;
+    NXEvent nullEvent = {NX_NULLEVENT, {0, 0}, 0, -1, 0};
+    enum { kNULLEventPostThrottle = 10 };
+    static io_connect_t io_connection = MACH_PORT_NULL;
+
+    if (io_connection == MACH_PORT_NULL) {
+	io_service_t service = IOServiceGetMatchingService(
+		kIOMasterPortDefault, IOServiceMatching(kIOHIDSystemClass));
+
+	if (service == MACH_PORT_NULL) {
+	    return;
+	}
+	kr = IOServiceOpen(service, mach_task_self(), kIOHIDParamConnectType,
+		&io_connection);
+	IOObjectRelease(service);
+	if (kr != KERN_SUCCESS) {
+	    return;
+	}
+    }
+    kr = IOHIDPostEvent(io_connection, NX_NULLEVENT, loc, &nullEvent.data,
+	    FALSE, 0, FALSE);
 }
+
+/*
+ * Local Variables:
+ * mode: objc
+ * c-basic-offset: 4
+ * fill-column: 79
+ * coding: utf-8
+ * End:
+ */
